<?php

// Prevent direct access to this file
if (!defined('ABSPATH')) {
    exit;
}


class SMAFFDKN_Referral_Discount {

    private $smaffdkn_coupon_code;

    public function __construct() {
        add_action('wp', array($this, 'capture_referral'));
        add_action('user_register', array($this, 'generate_discount_coupon'), 10, 1);
        add_filter('woocommerce_coupon_is_valid', array($this, 'force_coupon_validity'), 10, 3);
        add_filter('dokan_coupons_to_publish', array($this, 'allow_referral_coupons'), 10, 1);
    }

    public function capture_referral() {
        // Check if 'ref' parameter is present in the URL
        if (isset($_GET['ref'])) {
            // Verify the nonce
            if (isset($_GET['_wpnonce'])) {
                $nonce = sanitize_text_field(wp_unslash($_GET['_wpnonce']));
                if (wp_verify_nonce($nonce, 'capture_referral_nonce')) {
                    // Sanitize and validate the referral ID
                    $affiliate_id = intval(sanitize_text_field(wp_unslash($_GET['ref'])));

                    // Ensure affiliate ID is valid (e.g., greater than 0)
                    if ($affiliate_id > 0) {
                        // Set a cookie for the affiliate referral, valid for 24 hours
                        setcookie('smaffdkn_affiliate_ref', $affiliate_id, time() + 3600 * 24, COOKIEPATH, COOKIE_DOMAIN);
                    }
                } else {
                    // Nonce verification failed
                    wp_die('Nonce verification failed.');
                }
            }
        }
    }

    public function generate_discount_coupon($user_id) {
        if (isset($_COOKIE['smaffdkn_affiliate_ref'])) {
            $affiliate_id = intval($_COOKIE['smaffdkn_affiliate_ref']);

            $this->smaffdkn_coupon_code = 'REF-' . strtoupper(wp_generate_password(8, false));

            $coupon = new WC_Coupon();
            $coupon->set_code($this->smaffdkn_coupon_code);
            $coupon->set_discount_type('percent');
            $coupon->set_amount(get_option('smaffdkn_discount_rate', 10));
            $coupon->set_individual_use(true);
            $coupon->set_usage_limit(1);
            $coupon->set_date_expires(strtotime('+7 days'));
            $coupon->set_free_shipping(false);
            $coupon->set_minimum_amount(0);
            $coupon->set_usage_limit_per_user(1);
            $coupon->set_status('publish');

            $coupon_id = $coupon->save();

            if ($coupon_id && !is_wp_error($coupon_id)) {
                if (function_exists('wc_update_coupon_lookup')) {
                    wc_update_coupon_lookup($coupon_id);
                }

                $user_info = get_userdata($user_id);
                if ($user_info) {
                    $email = $user_info->user_email;
                    $this->send_coupon_email($email, $this->smaffdkn_coupon_code);
                }

                update_post_meta($coupon_id, 'affiliate_id', $affiliate_id);

                // Dokan specific meta
                update_post_meta($coupon_id, 'admin_coupons_enabled_for_vendor', 'yes');
                update_post_meta($coupon_id, 'coupon_commissions_type', 'default');
                update_post_meta($coupon_id, 'coupons_vendors_ids', '');
                update_post_meta($coupon_id, 'coupons_exclude_vendors_ids', '');
                update_post_meta($coupon_id, 'admin_shared_coupon_type', 'percentage');
                update_post_meta($coupon_id, 'admin_coupons_show_on_stores', 'no');

                $this->clear_caches();

                wp_cache_delete($coupon_id, 'posts');
                clean_post_cache($coupon_id);
            }
        }
    }

    private function clear_caches() {
        WC_Cache_Helper::get_transient_version('coupons', true);
        WC_Cache_Helper::invalidate_cache_group('coupons');
        wc_delete_shop_order_transients();
        wc_delete_product_transients();
        flush_rewrite_rules();
    }

    private function send_coupon_email($email, $coupon_code) {
        $site_title = get_bloginfo('name');
        $admin_email = get_option('admin_email');
        $discount_rate = get_option('smaffdkn_discount_rate', 10);
    
        /* translators: %1$d is the discount percentage, %2$s is the site title */
        $subject = sprintf(__('Your %1$d%% Discount Coupon from %2$s', 'smart-affiliate-for-dokan'), $discount_rate, $site_title);

        $message = $this->get_email_html_content($coupon_code, $discount_rate);
    
        $headers = array(
            'Content-Type: text/html; charset=UTF-8',
            "From: {$site_title} <{$admin_email}>",
            "Reply-To: {$admin_email}"
        );
    
        wp_mail($email, $subject, $message, $headers);
    }
    
    
    private function get_email_html_content($coupon_code, $discount_rate) {
        // Sanitize inputs early
        $coupon_code = esc_html($coupon_code);
        $discount_rate = absint($discount_rate);
        
        // Get and sanitize site information
        $site_url = esc_url(get_site_url());
        $site_title = esc_html(get_bloginfo('name'));
        $admin_email = esc_html(get_option('admin_email'));
        
        // Get logo attachment ID (you should store this in your options)
        $logo_id = get_theme_mod('custom_logo');
        
        // Load CSS styles
        ob_start();
        wp_print_styles('smaffdkn-email-template');
        $styles = ob_get_clean();

        // Prepare logo HTML with absolute URL
        if ($logo_id) {
            $image_attributes = wp_get_attachment_image_src($logo_id, 'full');
            if ($image_attributes) {
                $logo_absolute_url = $image_attributes[0];
                // Ensure URL is absolute
                if (!preg_match("~^(?:f|ht)tps?://~i", $logo_absolute_url)) {
                    $logo_absolute_url = site_url($logo_absolute_url);
                }
                $logo_html = wp_get_attachment_image(
                    $logo_id,
                    array(150, 150),
                    false,
                    array(
                        'class' => 'logo',
                        'style' => 'display: block; margin: 0 auto 20px auto; border: 0;',
                        'src' => $logo_absolute_url // Override src with absolute URL
                    )
                );
            }
        }
        
        // Fallback if no logo is found
        if (empty($logo_html)) {
            // Get WordPress default logo from media library
            $args = array(
                'post_type' => 'attachment',
                'name' => 'wordpress-logo',
                'posts_per_page' => 1,
                'post_status' => 'inherit'
            );
            
            $wp_logo_query = new WP_Query($args);
            
            if ($wp_logo_query->have_posts()) {
                $logo_id = $wp_logo_query->posts[0]->ID;
            } else {
                // If WordPress logo not in media library, add it
                $wp_logo_file = includes_url('images/wordpress-logo.png');
                
                // Get the path on server for this file using URL
                $wp_logo_path = str_replace(site_url('/'), ABSPATH, $wp_logo_file);
                
                if (file_exists($wp_logo_path)) {
                    $file_array = array(
                        'name' => 'wordpress-logo.png',
                        'tmp_name' => $wp_logo_path
                    );
                    
                    $logo_id = media_handle_sideload($file_array, 0);
                }
            }
            
            if (!empty($logo_id) && !is_wp_error($logo_id)) {
                $image_attributes = wp_get_attachment_image_src($logo_id, 'full');
                if ($image_attributes) {
                    $logo_absolute_url = $image_attributes[0];
                    if (!preg_match("~^(?:f|ht)tps?://~i", $logo_absolute_url)) {
                        $logo_absolute_url = site_url($logo_absolute_url);
                    }
                    $logo_html = wp_get_attachment_image(
                        $logo_id,
                        array(150, 150),
                        false,
                        array(
                            'class' => 'logo',
                            'style' => 'display: block; margin: 0 auto 20px auto; border: 0;',
                            'src' => $logo_absolute_url
                        )
                    );
                }
            }
            
            // Final fallback if everything else fails
            if (empty($logo_html)) {
                $attachment = array(
                    'guid' => admin_url('images/wordpress-logo.png'),
                    'post_mime_type' => 'image/png',
                    'post_title' => 'WordPress Logo',
                    'post_status' => 'inherit'
                );
                
                $logo_id = wp_insert_attachment($attachment);
                
                if (!is_wp_error($logo_id)) {
                    $logo_html = wp_get_attachment_image(
                        $logo_id,
                        array(150, 150),
                        false,
                        array(
                            'class' => 'logo',
                            'style' => 'display: block; margin: 0 auto 20px auto; border: 0;',
                            'src' => admin_url('images/wordpress-logo.png')
                        )
                    );
                }
            }
        }

        // HTML template with improved structure and compatibility
        $html = '<!DOCTYPE html>
        <html lang="en">
        <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <meta http-equiv="X-UA-Compatible" content="IE=edge">
            <title>' . $site_title . ' - Your Discount Coupon</title>
            ' . $styles . '
        </head>
        <body>
            <div class="container">
                <div class="header">
                    ' . $logo_html . '
                    <h1 style="margin: 0; font-size: 24px; color: #333333;">' . $site_title . '</h1>
                </div>
                
                <p>Thank you for registering! As a welcome gift, we\'re pleased to offer you a special discount.</p>
                
                <p>Here is your ' . $discount_rate . '% discount coupon:</p>
                
                <div class="coupon-code">' . $coupon_code . '</div>
                
                <p><strong>Use this coupon to get ' . $discount_rate . '% off your next purchase.</strong></p>
                
                <ul style="padding-left: 20px;">
                    <li>Valid for 7 days from now</li>
                    <li>Active immediately</li>
                    <li>Enter the code during checkout</li>
                </ul>
                
                <p>Happy shopping!</p>
                
                <div class="footer">
                    <p style="margin-bottom: 10px;">This email was sent from ' . $site_title . '.<br>
                    If you have any questions, please contact us at <a href="mailto:' . $admin_email . '">' . $admin_email . '</a></p>
                    <p style="margin: 0;"><a href="' . $site_url . '">' . $site_url . '</a></p>
                </div>
            </div>
        </body>
        </html>';
        
        return $html;
    }
    
    public function force_coupon_validity($is_valid, $coupon, $discount) {
        if (!$is_valid) {
            return $is_valid;
        }

        $coupon_id = $coupon->get_id();
        $coupon_code = $coupon->get_code();

        $enabled_for_vendor = get_post_meta($coupon_id, 'admin_coupons_enabled_for_vendor', true);
        if ($enabled_for_vendor !== 'yes') {
            return false;
        }

        $vendor_ids = get_post_meta($coupon_id, 'coupons_vendors_ids', true);
        $exclude_vendor_ids = get_post_meta($coupon_id, 'coupons_exclude_vendors_ids', true);

        if (!empty($vendor_ids) || !empty($exclude_vendor_ids)) {
            $product_authors = $this->get_cart_product_authors();
            
            if (!empty($vendor_ids)) {
                $vendor_ids = explode(',', $vendor_ids);
                if (!array_intersect($product_authors, $vendor_ids)) {
                    return false;
                }
            }

            if (!empty($exclude_vendor_ids)) {
                $exclude_vendor_ids = explode(',', $exclude_vendor_ids);
                if (array_intersect($product_authors, $exclude_vendor_ids)) {
                    return false;
                }
            }
        }

        $expiry_date = $coupon->get_date_expires();
        $usage_count = $coupon->get_usage_count();
        $usage_limit = $coupon->get_usage_limit();
        
        if ($expiry_date instanceof WC_DateTime) {
            $expiry_timestamp = $expiry_date->getTimestamp();
        } else {
            $expiry_timestamp = 0;
        }
        
        $current_time = current_time('timestamp');
        
        if ($expiry_timestamp && $expiry_timestamp <= $current_time) {
            return false;
        }
        
        if ($usage_limit !== 0 && $usage_count >= $usage_limit) {
            return false;
        }
        
        return true;
    }

    private function get_cart_product_authors() {
        $product_authors = array();
        foreach (WC()->cart->get_cart() as $cart_item) {
            $product_id = $cart_item['product_id'];
            $author_id = get_post_field('post_author', $product_id);
            if (!in_array($author_id, $product_authors)) {
                $product_authors[] = $author_id;
            }
        }
        return $product_authors;
    }

    public function allow_referral_coupons($coupons) {
        // Define a transient key
        $transient_key = 'smaffdkn_referral_coupons';
    
        // Attempt to get cached results
        $smaffdkn_referral_coupons = get_transient($transient_key);
    
        // If no cached data, perform the query and filter in PHP
        if ($smaffdkn_referral_coupons === false) {
            // Query for all published coupons
            $args = [
                'post_type' => 'shop_coupon',
                'post_status' => 'publish',
                'posts_per_page' => -1, // Retrieve all coupons
                'fields' => 'ids', // Retrieve only the post IDs
            ];
    
            $query = new WP_Query($args);
            $smaffdkn_referral_coupons = [];
    
            // Filter results by meta_key and title in PHP
            foreach ($query->posts as $post_id) {
                $title = get_the_title($post_id);
                $affiliate_id = get_post_meta($post_id, 'affiliate_id', true);
    
                // Check for affiliate_id meta and title starting with 'REF-'
                if (!empty($affiliate_id) && strpos($title, 'REF-') === 0) {
                    $smaffdkn_referral_coupons[] = $title;
                }
            }
    
            // Cache the results using transients for 12 hours
            set_transient($transient_key, $smaffdkn_referral_coupons, 12 * HOUR_IN_SECONDS);
        }
    
        // Merge and return the coupons
        return array_merge($coupons, $smaffdkn_referral_coupons);
    }
}

new SMAFFDKN_Referral_Discount();